function FindMotif_triangle_4_x(SIZE,R,datasize1, datasize2,topn, D1, D2)
%Find "topn" motifs by lowbound (using triangle inequality)
%tested on images in the folder that user choose
%normalized to SIZE*SIZE
%R = number of reference images
%datasize = number of samples

MAX = 999;
%INF = 999999;
outputfile = sprintf('NEW_top%d_R%d_size%d.txt',topn,R,SIZE);

%Read in images and perform the preprocess
if (nargin < 7)
    D1 = uigetdir('','Choose first directory that you want to find motif');
    D2 = uigetdir('','Choose second directory that you want to find motif');    
end

t=cputime;
flist1=dir(D1);
flist2=dir(D2);
% Read Images in First Directory
[images1 N1 names1]= ReadImagesDir(flist1, datasize1, SIZE, D1);
% Read Images in Second Directory
[images2 N2 names2]= ReadImagesDir(flist2, datasize2, SIZE, D2);
N=N1+N2;
images = cat(3,images1,images2);
names = cat(2,names1,names2);
t_read=cputime-t;

%set refenrence images
rand('twister',1000);
ref = ceil(N*rand(1,R));
bsf = MAX * ones(1,topn); %topn bsf
motifs = zeros(topn,2); %topn motif pairs

t = cputime;
%calculate the distance from all images to each ref image
Dist = ones(R,N);
for r = 1:R %from each ref image
    for n = 1:N %to each image
        if ref(r) == n
            Dist(r,n) = 0;
            continue;
        end
        Dist(r,n) = traditional(images(:,:,ref(r)), images(:,:,n));
        if (~IsDiffSet(ref(r),n,N1))
            continue;
        end

        if Dist(r,n) < bsf(topn) %insert the new found motifs
            replace_pos = topn;
            while replace_pos > 1 && Dist(r,n) < bsf(replace_pos-1)
                replace_pos = replace_pos - 1;
            end
            if replace_pos ~= topn
                for i = topn-1:-1:replace_pos %move down
                    bsf(i+1) = bsf(i);
                    motifs(i+1,1) = motifs(i,1);
                    motifs(i+1,2) = motifs(i,2);
                end
            end
            %insert the motifs
            bsf(replace_pos) = Dist(r,n);
            motifs(replace_pos,:) = [ref(r), n];
        end
    end
end
NumCalculation = (N-1) * R;
%find the row of Dist with max standard deviation
[tmp Z] = sort(std(Dist,0,2),'descend');
row = Z(1);
%get index based on Dist(row,:)
[lowbound index] = sort(Dist(row,:));
disp('Calculation to ref is done');
t_ref = cputime - t;
disp(t_ref);
disp(bsf);

t = cputime;
offset = 0; abandon = 0;
while abandon == 0
    offset = offset + 1; abandon = 1;
    for n = 2:N-offset %index(1) is one ref image, so don't need to calculate the dist from it to others
        reject = 0;
        for r = 1:R
            if abs(Dist(Z(r),index(n)) - Dist(Z(r),index(n+offset))) > bsf(topn) %if one lower bound > bsf
                reject = 1; break;  %then we don't need to calculate the exact distance
            elseif r == 1
                abandon = 0;
            end
        end
        if reject == 0
            if (IsDiffSet(index(n),index(n+offset),N1))
                exact = traditional(images(:,:,index(n)), images(:,:,index(n+offset))); %calculate the exact distance
            else
                %exact = INF;
                continue;
            end
            NumCalculation = NumCalculation + 1;
            if exact < bsf(topn) %insert the new found motifs
                replace_pos = topn;
                while replace_pos > 1 && exact < bsf(replace_pos-1)
                    replace_pos = replace_pos - 1;
                end
                if replace_pos ~= topn
                    for i = topn-1:-1:replace_pos %move down
                        bsf(i+1) = bsf(i);
                        motifs(i+1,1) = motifs(i,1);
                        motifs(i+1,2) = motifs(i,2);
                    end
                end
                bsf(replace_pos) = exact;
                motifs(replace_pos,:) = [index(n), index(n+offset)];
            end
        end
    end
    output = sprintf('%d\t%.2f',offset,cputime-t);
    disp(output);
     fid = fopen(outputfile,'a');
     fprintf(fid,'%s\r\n',output);
     fclose(fid);
end


%BruteForce = N*(N-1)/2;
BruteForce = N1*N2;
prunerate = (BruteForce - NumCalculation)/BruteForce;
output = sprintf('%d\t%.2f\t%f\t%.2f',N,cputime-t+t_ref,prunerate,t_read);
%disp(output);
fid = fopen(outputfile,'a');
fprintf(fid,'%s\r\n',output);
%fclose(fid);

%output motifs
% for i = 1:topn
%     fprintf(fid,'%s\t%s\r\n',names{motifs(i,1)},names{motifs(i,2)});
%     figure,subplot(1,2,1);imshow(I{motifs(i,1)});
%     subplot(1,2,2);imshow(I{motifs(i,2)});
% end

% figure,
% for i = 1:topn
%     fprintf(fid,'%s\t%s\t%f\r\n',names{motifs(i,1)},names{motifs(i,2)},bsf(i));
%     subplot(topn,2,2*i-1);imshow(I{motifs(i,1)});
%     subplot(topn,2,2*i);imshow(I{motifs(i,2)});
% end

%%figure,
for i = 1:topn
    fprintf(fid,'%s\t%s\t%f\r\n',names{motifs(i,1)},names{motifs(i,2)},bsf(i));
    %disp(sprintf('%s\t%s\t%f\r\n',names{motifs(i,1)},names{motifs(i,2)},bsf(i)));
% %     j = mod(i,5);
% %     if j ~= 0
% %         subplot(5,2,2*j-1);imshow(I{motifs(i,1)});
% %         subplot(5,2,2*j);imshow(I{motifs(i,2)});
% %     else
% %         subplot(5,2,9);imshow(I{motifs(i,1)});
% %         subplot(5,2,10);imshow(I{motifs(i,2)});
% %         figure,
% %     end
end

fclose(fid);

end


function Exactdist = traditional(img_train, img_test)
%Calculat the exact missed by traditinal method   
SIZEY = size(img_train,1);
SIZEX = size(img_train,2);

[idy, idx] = find(img_test == 0);
pointsnum_test = size(idy,1);
%calculate the center
p = [idx, idy]; % x and y coordinates
center_test = round(mean(p));
%compute the voting vector dv for each point 
cmat = repmat(center_test, pointsnum_test, 1);
dv = cmat - p;

[idy, idx] = find(img_train == 0);
pointsnum_train = size(idy,1);
q = [idx, idy]; 

vote_space = zeros(SIZEY,SIZEX);
for n = 1 : pointsnum_train %for each black pixel in train image
    pmat = repmat(q(n,:), pointsnum_test, 1);
    %vote for the center
    vote_points = pmat + dv;
    id = find(vote_points(:,1) >= 1 & vote_points(:,1) <= SIZEX &...
              vote_points(:,2) >= 1 & vote_points(:,2) <= SIZEY);
    if ~isempty(id)      
      pid = vote_points(id,2) + (vote_points(id,1) - 1) * SIZEY;
      vote_space(pid) = vote_space(pid) + 1;
    end
end   

%ExactMissed = pointsnum_test - max(vote_space(:));
% if pointsnum_test < pointsnum_train   
%     ExactMissed = (1/(pointsnum_test - ExactMissed)) * sqrt(pointsnum_train/pointsnum_test);
% else
%     ExactMissed = 1/(pointsnum_test - ExactMissed);
% end

Exactdist = (pointsnum_train + pointsnum_test)/2 -  max(vote_space(:));
%Exactdist = 1/2 -  max(vote_space(:))/(pointsnum_train + pointsnum_test);

end



function [images N names] = ReadImagesDir(flist, datasize, SIZE, D)
fnum = length(flist);
i = 1;
j = 0;
images = ones(SIZE,SIZE,datasize,'uint8');
I = cell(1,datasize);
names = cell(1,datasize);
while i <= fnum    
    if ~flist(i).isdir && ~strcmp(flist(i).name,'Thumbs.db')
        j = j + 1;
        names{j} = flist(i).name;
        I{j} = imread(sprintf('%s/%s',D,flist(i).name));  
        try %if I{j} is not binary
            image_bw = ~im2bw(I{j},graythresh(I{j}));
        catch
            image_bw = ~I{j};
        end
        %reduce the printing noise
        image_d = bwmorph(image_bw,'dilate');
        image_m = bwmorph(image_d,'majority');
        
        image_thin = bwmorph(image_m,'thin',3);%Thinning
        image_thin = bwmorph(image_thin,'remove'); %Removes interior pixels
        images(:,:,j) = normalize_img(~image_thin,SIZE); %normalize    
    end
    i = i + 1;
end
N = j;
disp([num2str(N) ' images read.']);
end

function flag = IsDiffSet(m,n,N1)
if (((m<=N1)&&(n>N1)) || ((m>N1)&&(n<=N1)))
    flag = true;
else
    flag = false;
end
end